@@ -38,11 +38,12 @@
 GtkListStore *
 gd_create_list_store (void)
 {
-  return gtk_list_store_new (4,
+  return gtk_list_store_new (5,
                              G_TYPE_STRING, // URN
                              G_TYPE_STRING, // TITLE
                              G_TYPE_STRING, // AUTHOR
-                             GDK_TYPE_PIXBUF); // ICON
+                             GDK_TYPE_PIXBUF, // ICON
+                             G_TYPE_LONG); // MTIME
 }
 
 void
@@ -51,13 +52,15 @@ gd_store_set (GtkListStore *store,
               const gchar *urn,
               const gchar *title,
               const gchar *author,
-              GdkPixbuf *icon)
+              GdkPixbuf *icon,
+              glong mtime)
 {
   gtk_list_store_set (store, iter,
                       0, urn,
                       1, title,
                       2, author,
                       3, icon,
+                      4, mtime,
                       -1);
 }
 
@@ -433,3 +436,89 @@ gd_filename_strip_extension (const char * filename_with_extension)
 
 	return filename;
 }
+
+/**
+ * gd_time_val_from_iso8601:
+ * @string:
+ * @timeval: (out):
+ *
+ * Returns:
+ */
+gboolean
+gd_time_val_from_iso8601 (const gchar *string,
+                          GTimeVal *timeval)
+{
+  return g_time_val_from_iso8601 (string, timeval);
+}
+
+#define _BG_MIN_SIZE 20
+#define _EMBLEM_MIN_SIZE 8
+
+/**
+ * gd_create_symbolic_icon:
+ * @name:
+ *
+ * Returns: (transfer full):
+ */
+GIcon *
+gd_create_symbolic_icon (const gchar *name,
+                         gint base_size)
+{
+  gchar *symbolic_name;
+  GIcon *icon, *retval = NULL;
+  cairo_surface_t *surface;
+  cairo_t *cr;
+  GtkStyleContext *style;
+  GtkWidgetPath *path;
+  GdkPixbuf *pixbuf;
+  GtkIconTheme *theme;
+  GtkIconInfo *info;
+  gint bg_size;
+  gint emblem_size;
+  gint total_size;
+
+  total_size = base_size / 2;
+  bg_size = MAX (total_size / 2, _BG_MIN_SIZE);
+  emblem_size = MAX (bg_size - 8, _EMBLEM_MIN_SIZE);
+
+  surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, total_size, total_size);
+  cr = cairo_create (surface);
+
+  style = gtk_style_context_new ();
+
+  path = gtk_widget_path_new ();
+  gtk_widget_path_append_type (path, GTK_TYPE_ICON_VIEW);
+  gtk_style_context_set_path (style, path);
+  gtk_widget_path_unref (path);
+
+  gtk_style_context_add_class (style, "documents-icon-bg");
+
+  gtk_render_background (style, cr, (total_size - bg_size) / 2, (total_size - bg_size) / 2, bg_size, bg_size);
+
+  symbolic_name = g_strconcat (name, "-symbolic", NULL);
+  icon = g_themed_icon_new (symbolic_name);
+
+  theme = gtk_icon_theme_get_default();
+  info = gtk_icon_theme_lookup_by_gicon (theme, icon, emblem_size,
+                                         GTK_ICON_LOOKUP_FORCE_SIZE);
+  pixbuf = gtk_icon_info_load_symbolic_for_context (info, style, NULL, NULL);
+
+  gtk_icon_info_free (info);
+  g_object_unref (icon);
+  g_free (symbolic_name);
+
+  if (pixbuf == NULL)
+    goto out;
+
+  gtk_render_icon (style, cr, pixbuf, (total_size - emblem_size) / 2,  (total_size - emblem_size) / 2);
+  g_object_unref (pixbuf);
+
+  retval = G_ICON (gdk_pixbuf_get_from_surface (surface, 0, 0, total_size, total_size));
+
+ out:
+  g_object_unref (style);
+  cairo_surface_destroy (surface);
+  cairo_destroy (cr);
+
+  return retval;
+}
